"""pysmartmeter - Collect data from Hichi Smartmeter and expose it via MQTT"""

from importlib.metadata import version


__version__ = version('pysmartmeter')
__author__ = 'Jens Diemer <pysmartmeter@jensdiemer.de>'
