# from: https://github.com/home-assistant/core/blob/dev/homeassistant/const.py
CONF_DEVICE = "device"
CONF_DEVICE_CLASS = "device_class"

# from: https://github.com/home-assistant/core/blob/dev/homeassistant/components/mqtt/const.py
CONF_STATE_TOPIC = "state_topic"
DEFAULT_PREFIX = "homeassistant"  # Discovery prefix
